"""Converters for the WhiteBit Private API."""

from aiowhitebit.converters.private.v4 import (
    convert_executed_deals_to_dto,
    convert_executed_orders_by_market_to_dto,
    convert_executed_orders_to_dto,
    convert_active_orders_to_dto,
    convert_cancel_order_to_dto,
    convert_order_response_to_dto,
    convert_get_trading_balance_to_dto,
)
