"""Public API v2 models."""

from aiowhitebit.models.public.v2.request import (
    RecentTradesRequest,
    OrderDepthRequest,
)
from aiowhitebit.models.public.v2.response import (
    Market,
    MarketInfo,
    Ticker,
    Tickers,
    RecentTrade,
    RecentTrades,
    Fee,
    FeeResponse,
    Asset,
    AssetStatus,
    OrderDepthV2,
)
